#!/usr/bin/env python
# License: GPLv3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>

import operator
import unittest

from calibre.utils.search_query_parser import Parser, SearchQueryParser


class Tester(SearchQueryParser):
    texts = {
        1: ['Eugenie Grandet', 'Honor\xe9 de Balzac', 'manybooks.net', 'lrf'],
        2: ['Fanny Hill', 'John Cleland', 'manybooks.net', 'lrf'],
        3: ['Persuasion', 'Jane Austen', 'manybooks.net', 'lrf'],
        4: ['Psmith, Journalist', 'P. G. Wodehouse', 'Some Publisher', 'lrf'],
        5: ['The Complete Works of William Shakespeare', 'William Shakespeare', 'manybooks.net', 'lrf'],
        6: ['The History of England, Volume I', 'David Hume', 'manybooks.net', 'lrf'],
        7: ['Someone Comes to Town, Someone Leaves Town', 'Cory Doctorow', 'Tor Books', 'lrf'],
        8: ['Stalky and Co.', 'Rudyard Kipling', 'manybooks.net', 'lrf'],
        9: ['A Game of Thrones', 'George R. R. Martin', None, 'lrf,rar'],
        10: ['A Clash of Kings', 'George R. R. Martin', None, 'lrf,rar'],
        11: ['A Storm of Swords', 'George R. R. Martin', None, 'lrf,rar'],
        12: ['Biggles - Pioneer Air Fighter', 'W. E. Johns', None, 'lrf,rtf'],
        13: ['Biggles of the Camel Squadron', 'W. E. Johns', 'London:Thames, (1977)', 'lrf,rtf'],
        14: ['A Feast for Crows', 'George R. R. Martin', None, 'lrf,rar'],
        15: ['Cryptonomicon', 'Neal Stephenson', None, 'lrf,rar'],
        16: ['Quicksilver', 'Neal Stephenson', None, 'lrf,zip'],
        17: ['The Comedies of William Shakespeare', 'William Shakespeare', None, 'lrf'],
        18: ['The Histories of William Shakespeare', 'William Shakespeare', None, 'lrf'],
        19: ['The Tragedies of William Shakespeare', 'William Shakespeare', None, 'lrf'],
        20: ['An Ideal Husband', 'Oscar Wilde', 'manybooks.net', 'lrf'],
        21: ['Flight of the Nighthawks', 'Raymond E. Feist', None, 'lrf,rar'],
        22: ['Into a Dark Realm', 'Raymond E. Feist', None, 'lrf,rar'],
        23: ['The Sundering', 'Walter Jon Williams', None, 'lrf,rar'],
        24: ['The Praxis', 'Walter Jon Williams', None, 'lrf,rar'],
        25: ['Conventions of War', 'Walter Jon Williams', None, 'lrf,rar'],
        26: ['Banewreaker', 'Jacqueline Carey', None, 'lrf,rar'],
        27: ['Godslayer', 'Jacqueline Carey', None, 'lrf,rar'],
        28: ["Kushiel's Scion", 'Jacqueline Carey', None, 'lrf,rar'],
        29: ['Underworld', 'Don DeLillo', None, 'lrf,rar'],
        30: [
            'Genghis Khan and The Making of the Modern World',
            'Jack Weatherford Orc',
            'Three Rivers Press',
            'lrf,zip',
        ],
        31: ['The Best and the Brightest', 'David Halberstam', 'Modern Library', 'lrf,zip'],
        32: ['The Killer Angels', 'Michael Shaara', None, 'html,lrf'],
        33: ['Band Of Brothers', 'Stephen E Ambrose', None, 'lrf,txt'],
        34: ['The Gates of Rome', 'Conn Iggulden', None, 'lrf,rar'],
        35: ['The Death of Kings', 'Conn Iggulden', 'Bantam Dell', 'lit,lrf'],
        36: ['The Field of Swords', 'Conn Iggulden', None, 'lrf,rar'],
        37: ['Masterman Ready', 'Marryat, Captain Frederick', None, 'lrf'],
        38: ['With the Lightnings', 'David Drake', 'Baen Publishing Enterprises', 'lit,lrf'],
        39: ['Lt. Leary, Commanding', 'David Drake', 'Baen Publishing Enterprises', 'lit,lrf'],
        40: ['The Far Side of The Stars', 'David Drake', 'Baen Publishing Enterprises', 'lrf,rar'],
        41: ['The Way to Glory', 'David Drake', 'Baen Publishing Enterprises', 'lrf,rar'],
        42: ['Some Golden Harbor', 'David Drake', 'Baen Books', 'lrf,rar'],
        43: ['Harry Potter And The Half-Blood Prince', 'J. K. Rowling', None, 'lrf,rar'],
        44: ['Harry Potter and the Order of the Phoenix', 'J. K. Rowling', None, 'lrf,rtf'],
        45: ['The Stars at War', 'David Weber , Steve White', None, 'lrf,rtf'],
        46: ['The Stars at War II', 'Steve White', 'Baen Publishing Enterprises', 'lrf,rar'],
        47: ['Exodus', 'Steve White,Shirley Meier', 'Baen Books', 'lrf,rar'],
        48: ['Harry Potter and the Goblet of Fire', 'J. K. Rowling', None, 'lrf,rar'],
        49: ['Harry Potter and the Prisoner of Azkaban', 'J. K. Rowling', None, 'lrf,rtf'],
        50: ['Harry Potter and the Chamber of Secrets', 'J. K. Rowling', None, 'lit,lrf'],
        51: ['Harry Potter and the Deathly Hallows', 'J.K. Rowling', None, 'lit,lrf,pdf'],
        52: ["His Majesty's Dragon", 'Naomi Novik', None, 'lrf,rar'],
        53: ['Throne of Jade', 'Naomi Novik', 'Del Rey', 'lit,lrf'],
        54: ['Black Powder War', 'Naomi Novik', 'Del Rey', 'lrf,rar'],
        55: ['War and Peace', 'Leo Tolstoy', 'gutenberg.org', 'lrf,txt'],
        56: ['Anna Karenina', 'Leo Tolstoy', 'gutenberg.org', 'lrf,txt'],
        57: ['A Shorter History of Rome', 'Eugene Lawrence,Sir William Smith', 'gutenberg.org', 'lrf,zip'],
        58: ['The Name of the Rose', 'Umberto Eco', None, 'lrf,rar'],
        71: ["Wind Rider's Oath", 'David Weber', 'Baen', 'lrf'],
        74: ['Rally Cry', 'William R Forstchen', None, 'htm,lrf'],
        86: ['Empire of Ivory', 'Naomi Novik', None, 'lrf,rar'],
        87: ["Renegade's Magic", 'Robin Hobb', None, 'lrf,rar'],
        89: ['Master and commander', "Patrick O'Brian", 'Fontana,\n1971', 'lit,lrf'],
        91: ['A Companion to Wolves', 'Sarah Monette,Elizabeth Beär', None, 'lrf,rar'],
        92: ['The Lions of al-Rassan', 'Guy Gavriel Kay', 'Eos', 'lit,lrf'],
        93: ['Gardens of the Moon', 'Steven Erikson', 'Tor Fantasy', 'lit,lrf'],
        95: ['The Master and Margarita', 'Mikhail Bulgakov', 'N.Y. : Knopf, 1992.', 'lrf,rtf'],
        120: ['Deadhouse Gates', 'Steven Erikson', 'London : Bantam Books, 2001.', 'lit,lrf'],
        121: ['Memories of Ice', 'Steven Erikson', 'Bantam Books', 'lit,lrf'],
        123: ['House of Chains', 'Steven Erikson', 'Bantam Books', 'lit,lrf'],
        125: ['Midnight Tides', 'Steven Erikson', 'Bantam Books', 'lit,lrf'],
        126: ['The Bonehunters', 'Steven Erikson', 'Bantam Press', 'lit,lrf'],
        129: [
            'Guns, germs, and steel: the fates of human societies',
            'Jared Diamond',
            'New York : W.W. Norton, c1997.',
            'lit,lrf',
        ],
        136: ['Wildcards', 'George R. R. Martin', None, 'html,lrf'],
        138: ['Off Armageddon Reef', 'David Weber', 'Tor Books', 'lit,lrf'],
        144: ['Atonement', 'Ian McEwan', 'New York : Nan A. Talese/Doubleday, 2002.', 'lrf,rar'],
        146: ['1632', 'Eric Flint', 'Baen Books', 'lit,lrf'],
        147: ['1633', 'David Weber,Eric Flint,Dru Blair', 'Baen', 'lit,lrf'],
        148: ['1634: The Baltic War', 'David Weber,Eric Flint', 'Baen', 'lit,lrf'],
        150: ['The Dragonbone Chair', 'Tad Williams', 'DAW Trade', 'lrf,rtf'],
        152: ['The Little Book That Beats the Market', 'Joel Greenblatt', 'Wiley', 'epub,lrf'],
        153: ['Pride of Carthage', 'David Anthony Durham', 'Anchor', 'lit,lrf'],
        154: ['Stone of farewell', 'Tad Williams', 'New York : DAW Books, 1990.', 'lrf,txt'],
        166: ['American Gods', 'Neil Gaiman', 'HarperTorch', 'lit,lrf'],
        176: ['Pillars of the Earth', 'Ken Follett', 'New American Library', 'lit,lrf'],
        182: ['The Eye of the world', 'Robert Jordan', 'New York : T. Doherty Associates, c1990.', 'lit,lrf'],
        188: ['The Great Hunt', 'Robert Jordan', 'ATOM', 'lrf,zip'],
        189: ['The Dragon Reborn', 'Robert Jordan', None, 'lit,lrf'],
        190: ['The Shadow Rising', 'Robert Jordan', None, 'lit,lrf'],
        191: ['The Fires of Heaven', 'Robert Jordan', 'Time Warner Books Uk', 'lit,lrf'],
        216: ['Lord of chaos', 'Robert Jordan', 'New York : TOR, c1994.', 'lit,lrf'],
        217: ['A Crown of Swords', 'Robert Jordan', None, 'lit,lrf'],
        236: ['The Path of Daggers', 'Robert Jordan', None, 'lit,lrf'],
        238: ['The Client', 'John Grisham', 'New York : Island, 1994, c1993.', 'lit,lrf'],
        240: ["Winter's Heart", 'Robert Jordan', None, 'lit,lrf'],
        242: ['In the Beginning was the Command Line', 'Neal Stephenson', None, 'lrf,txt'],
        249: ['Crossroads of Twilight', 'Robert Jordan', None, 'lit,lrf'],
        251: ['Caves of Steel', 'Isaac Asimov', 'Del Rey', 'lrf,zip'],
        253: ["Hunter's Run", 'George R. R. Martin,Gardner Dozois,Daniel Abraham', 'Eos', 'lrf,rar'],
        257: ['Knife of Dreams', 'Robert Jordan', None, 'lit,lrf'],
        258: ['Saturday', 'Ian McEwan', 'London : Jonathan Cape, 2005.', 'lrf,txt'],
        259: [
            'My name is Red',
            'Orhan Pamuk; translated from the Turkish by Erdağ G\xf6knar',
            'New York : Alfred A. Knopf, 2001.',
            'lit,lrf',
        ],
        265: ['Harbinger', 'David Mack', 'Star Trek', 'lit,lrf'],
        267: ['Summon the Thunder', 'Dayton Ward,Kevin Dilmore', 'Pocket Books', 'lit,lrf'],
        268: ['Shalimar the Clown', 'Salman Rushdie', 'New York : Random House, 2005.', 'lit,lrf'],
        269: ['Reap the Whirlwind', 'David Mack', 'Star Trek', 'lit,lrf'],
        272: ['Mistborn', 'Brandon Sanderson', 'Tor Fantasy', 'lrf,rar'],
        273: ['The Thousandfold Thought', 'R. Scott Bakker', 'Overlook TP', 'lrf,rtf'],
        276: ['Elantris', 'Brandon Sanderson', 'New York : Tor, 2005.', 'lrf,rar'],
        291: ['Sundiver', 'David Brin', 'New York : Bantam Books, 1995.', 'lit,lrf'],
        299: ['Imperium', 'Robert Harris', 'Arrow', 'lrf,rar'],
        300: ['Startide Rising', 'David Brin', 'Bantam', 'htm,lrf'],
        301: ['The Uplift War', 'David Brin', 'Spectra', 'lit,lrf'],
        304: ['Brightness Reef', 'David Brin', 'Orbit', 'lrf,rar'],
        305: ["Infinity's Shore", 'David Brin', 'Spectra', 'txt'],
        306: ["Heaven's Reach", 'David Brin', 'Spectra', 'lrf,rar'],
        325: ["Foundation's Triumph", 'David Brin', 'Easton Press', 'lit,lrf'],
        327: ['I am Charlotte Simmons', 'Tom Wolfe', 'Vintage', 'htm,lrf'],
        335: ['The Currents of Space', 'Isaac Asimov', None, 'lit,lrf'],
        340: ['The Other Boleyn Girl', 'Philippa Gregory', 'Touchstone', 'lit,lrf'],
        341: ["Old Man's War", 'John Scalzi', 'Tor', 'htm,lrf'],
        342: ['The Ghost Brigades', 'John Scalzi', 'Tor Science Fiction', 'html,lrf'],
        343: ['The Last Colony', 'John S"calzi', 'Tor Books', 'html,lrf'],
        344: ['Gossip Girl', 'Cecily von Ziegesar', 'Warner Books', 'lrf,rtf'],
        347: ['Little Brother', 'Cory Doctorow', 'Tor Teen', 'lrf'],
        348: ['The Reality Dysfunction', 'Peter F. Hamilton', 'Pan MacMillan', 'lit,lrf'],
        353: ['A Thousand Splendid Suns', 'Khaled Hosseini', 'Center Point Large Print', 'lit,lrf'],
        354: ['Amsterdam', 'Ian McEwan', 'Anchor', 'lrf,txt'],
        355: ['The Neutronium Alchemist', 'Peter F. Hamilton', 'Aspect', 'lit,lrf'],
        356: ['The Naked God', 'Peter F. Hamilton', 'Aspect', 'lit,lrf'],
        421: ['A Shadow in Summer', 'Daniel Abraham', 'Tor Fantasy', 'lrf,rar'],
        427: ['Lonesome Dove', 'Larry M\\cMurtry', None, 'lit,lrf'],
        440: ['Ghost', 'John Ringo', 'Baen', 'lit,lrf'],
        441: ['Kildar', 'John Ringo', 'Baen', 'lit,lrf'],
        443: ['Hidden Empire ', 'Kevin J. Anderson', 'Aspect', 'lrf,rar'],
        444: ['The Gun Seller', 'Hugh Laurie', 'Washington Square Press', 'lrf,rar'],
    }

    tests = {
        'Dysfunction': {348},
        'title:Dysfunction': {348},
        'Title:Dysfunction': {348},
        'title:Dysfunction OR author:Laurie': {348, 444},
        '(tag:txt or tag:pdf)': {33, 258, 354, 305, 242, 51, 55, 56, 154},
        '(tag:txt OR tag:pdf) and author:Tolstoy': {55, 56},
        'Tolstoy txt': {55, 56},
        'Hamilton Amsterdam': set(),
        'Beär': {91},
        'dysfunc or tolstoy': {348, 55, 56},
        'tag:txt AND NOT tolstoy': {33, 258, 354, 305, 242, 154},
        'not tag:lrf': {305},
        'london:thames': {13},
        'publisher:london:thames': {13},
        '"(1977)"': {13},
        'jack weatherford orc': {30},
        'S\\"calzi': {343},
        'author:S\\"calzi': {343},
        '"S\\"calzi"': {343},
        'M\\\\cMurtry': {427},
        'author:Tolstoy (tag:txt OR tag:pdf)': {55, 56},
    }
    fields = {'title': 0, 'author': 1, 'publisher': 2, 'tag': 3}

    _universal_set = set(texts.keys())

    def universal_set(self):
        return self._universal_set

    def get_matches(self, location, query, candidates=None):
        location = location.lower()
        if location in self.fields.keys():
            getter = operator.itemgetter(self.fields[location])
        elif location == 'all':

            def getter(y):
                return ''.join(x or '' for x in y)

        else:

            def getter(x):
                return ''

        if not query:
            return set()
        query = query.lower()
        if candidates:
            return {key for key, val in self.texts.items() if key in candidates and query and query in getattr(getter(val), 'lower', lambda: '')()}
        else:
            return {key for key, val in self.texts.items() if query and query in getattr(getter(val), 'lower', lambda: '')()}

    def run_tests(self, ae):
        for query in self.tests.keys():
            res = self.parse(query)
            ae(self.tests[query], res, f'Failed for query: {query}')


class TestSQP(unittest.TestCase):
    def do_test(self, optimize=False):
        tester = Tester(
            [
                'authors',
                'author',
                'series',
                'formats',
                'format',
                'publisher',
                'rating',
                'tags',
                'tag',
                'comments',
                'comment',
                'cover',
                'isbn',
                'ondevice',
                'pubdate',
                'size',
                'date',
                'title',
                '#read',
                'all',
                'search',
            ],
            test=True,
            optimize=optimize,
        )
        tester.run_tests(self.assertEqual)

    def test_sqp_optimized(self):
        self.do_test(True)

    def test_sqp_unoptimized(self):
        self.do_test(False)

    def test_sqp_tokenizer(self):
        p = Parser()

        def tokens(*a):
            ans = []
            for i in range(0, len(a), 2):
                ans.append(({'O': Parser.OPCODE, 'W': Parser.WORD, 'Q': Parser.QUOTED_WORD}[a[i]], a[i + 1]))
            return ans

        def t(query, *a):
            self.assertEqual(tokens(*a), p.tokenize(query))

        t('xxx', 'W', 'xxx')
        t('"a \\" () b"', 'Q', 'a " () b')
        t('"a“b"', 'Q', 'a“b')
        t('"a”b"', 'Q', 'a”b')
        # docstring tests
        t(r'"""a\1b"""', 'W', r'a\1b')
        t(r'("""a\1b""" AND """c""" OR d)', 'O', '(', 'W', r'a\1b', 'W', 'AND', 'W', 'c', 'W', 'OR', 'W', 'd', 'O', ')')
        t(r'template:="""a\1b"""', 'W', r'template:=a\1b')
        t('template:="""a\nb"""', 'W', 'template:=a\nb')
        t(r'template:"""=a\1b"""', 'W', r'template:=a\1b')
        t(r'template:"""program: return ("\"1\"")#@#n:1"""', 'W', r'template:program: return ("\"1\"")#@#n:1')


def find_tests():
    return unittest.defaultTestLoader.loadTestsFromTestCase(TestSQP)


class TestRunner(unittest.main):
    def createTests(self):
        self.test = find_tests()


def run(verbosity=4):
    TestRunner(verbosity=verbosity, exit=False)


if __name__ == '__main__':
    run()



# License: GPLv3 Copyright: 2008, Kovid Goyal kovid@kovidgoyal.net

"""
Convert an ODT file into a Open Ebook
"""

from calibre.customize.conversion import InputFormatPlugin
from calibre.utils.localization import _


class ODTInput(InputFormatPlugin):
    name = 'ODT Input'
    author = 'Kovid Goyal'
    description = _('Convert ODT (LibreOffice) files to HTML')
    file_types = {'odt'}
    commit_name = 'odt_input'

    def convert(self, stream, options, file_ext, log, accelerators):
        from calibre.ebooks.odt.input import Extract

        return Extract()(stream, '.', log)



# autogenerated by __main__.py do not edit
name_map = {
    'PYQT_WEBENGINE_VERSION': 'PyQt6.QtWebEngineCore',
    'PYQT_WEBENGINE_VERSION_STR': 'PyQt6.QtWebEngineCore',
    'QWebChannel': 'PyQt6.QtWebChannel',
    'QWebChannelAbstractTransport': 'PyQt6.QtWebChannel',
    'QWebEngineCertificateError': 'PyQt6.QtWebEngineCore',
    'QWebEngineClientCertificateSelection': 'PyQt6.QtWebEngineCore',
    'QWebEngineClientCertificateStore': 'PyQt6.QtWebEngineCore',
    'QWebEngineClientHints': 'PyQt6.QtWebEngineCore',
    'QWebEngineContextMenuRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineCookieStore': 'PyQt6.QtWebEngineCore',
    'QWebEngineDesktopMediaRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineDownloadRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineExtensionInfo': 'PyQt6.QtWebEngineCore',
    'QWebEngineExtensionManager': 'PyQt6.QtWebEngineCore',
    'QWebEngineFileSystemAccessRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineFindTextResult': 'PyQt6.QtWebEngineCore',
    'QWebEngineFrame': 'PyQt6.QtWebEngineCore',
    'QWebEngineFullScreenRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineGlobalSettings': 'PyQt6.QtWebEngineCore',
    'QWebEngineHistory': 'PyQt6.QtWebEngineCore',
    'QWebEngineHistoryItem': 'PyQt6.QtWebEngineCore',
    'QWebEngineHistoryModel': 'PyQt6.QtWebEngineCore',
    'QWebEngineHttpRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineLoadingInfo': 'PyQt6.QtWebEngineCore',
    'QWebEngineNavigationRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineNewWindowRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineNotification': 'PyQt6.QtWebEngineCore',
    'QWebEnginePage': 'PyQt6.QtWebEngineCore',
    'QWebEnginePermission': 'PyQt6.QtWebEngineCore',
    'QWebEngineProfile': 'PyQt6.QtWebEngineCore',
    'QWebEngineProfileBuilder': 'PyQt6.QtWebEngineCore',
    'QWebEngineQuotaRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineRegisterProtocolHandlerRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineScript': 'PyQt6.QtWebEngineCore',
    'QWebEngineScriptCollection': 'PyQt6.QtWebEngineCore',
    'QWebEngineSettings': 'PyQt6.QtWebEngineCore',
    'QWebEngineUrlRequestInfo': 'PyQt6.QtWebEngineCore',
    'QWebEngineUrlRequestInterceptor': 'PyQt6.QtWebEngineCore',
    'QWebEngineUrlRequestJob': 'PyQt6.QtWebEngineCore',
    'QWebEngineUrlScheme': 'PyQt6.QtWebEngineCore',
    'QWebEngineUrlSchemeHandler': 'PyQt6.QtWebEngineCore',
    'QWebEngineView': 'PyQt6.QtWebEngineWidgets',
    'QWebEngineWebAuthPinRequest': 'PyQt6.QtWebEngineCore',
    'QWebEngineWebAuthUxRequest': 'PyQt6.QtWebEngineCore',
    'QtWebChannel': 'PyQt6.QtWebChannel',
    'QtWebEngineCore': 'PyQt6.QtWebEngineCore',
    'QtWebEngineWidgets': 'PyQt6.QtWebEngineWidgets',
    'qWebEngineChromiumSecurityPatchVersion': 'PyQt6.QtWebEngineCore',
    'qWebEngineChromiumVersion': 'PyQt6.QtWebEngineCore',
    'qWebEngineGetDomainAndRegistry': 'PyQt6.QtWebEngineCore',
    'qWebEngineProcessName': 'PyQt6.QtWebEngineCore',
    'qWebEngineVersion': 'PyQt6.QtWebEngineCore',
}
module_names = frozenset(('QtWebEngineCore', 'QtWebEngineWidgets', 'QtWebChannel'))



#!/usr/bin/env python3
# License: GPLv3 Copyright: 2012, Kovid Goyal <kovid at kovidgoyal.net>

import os
import shutil
import subprocess

base = os.path.dirname(os.path.abspath(__file__))

os.chdir('/tmp')
if os.path.exists('libmtp'):
    shutil.rmtree('libmtp')
subprocess.check_call(['git', 'clone', '--depth=1', 'git://git.code.sf.net/p/libmtp/code', 'libmtp'])
for x in ('src/music-players.h', 'src/device-flags.h'):
    shutil.copyfile('libmtp/' + x, os.path.join(base, os.path.basename(x)))



#!/usr/bin/env python
# License: GPLv3 Copyright: 2025, Kovid Goyal <kovid@kovidgoyal.net>

from functools import partial

from calibre.gui2 import config, gprefs
from calibre.gui2.preferences import LazyConfigWidgetBase
from calibre.gui2.preferences.look_feel_tabs import DisplayedFields, export_layout, import_layout, move_field_down, move_field_up, reset_layout
from calibre.gui2.preferences.look_feel_tabs.tb_display_ui import Ui_Form


class TBDisplayedFields(DisplayedFields):
    # The code in this class depends on the fact that the tag browser is
    # initialized before this class is instantiated.

    def __init__(self, db, parent=None, category_icons=None):
        DisplayedFields.__init__(self, db, parent, category_icons=category_icons)
        from calibre.gui2.ui import get_gui

        self.gui = get_gui(fail_if_absent=True)

    def initialize(self, use_defaults=False, pref_data_override=None):
        from calibre.gui2.tag_browser.model import TagsModel

        tv = self.gui.tags_view
        model = tv.model()
        assert isinstance(model, TagsModel)
        cat_ord = model.get_ordered_categories(use_defaults=use_defaults, pref_data_override=pref_data_override)
        if use_defaults:
            hc = []
            self.changed = True
        elif pref_data_override:
            hc = [k for k, v in pref_data_override if not v]
            self.changed = True
        else:
            hc = tv.hidden_categories

        self.beginResetModel()
        self.fields = [[x, x not in hc] for x in cat_ord]
        self.endResetModel()

    def commit(self):
        if self.changed:
            self.db.prefs.set('tag_browser_hidden_categories', sorted(k for k, v in self.fields if not v))
            self.db.prefs.set('tag_browser_category_order', [k for k, v in self.fields])


class TbDisplayTab(LazyConfigWidgetBase, Ui_Form):
    def genesis(self, gui):
        self.gui = gui
        r = self.register
        r('tag_browser_old_look', gprefs)
        r('tag_browser_hide_empty_categories', gprefs)
        r('tag_browser_always_autocollapse', gprefs)
        r('tag_browser_restore_tree_expansion', gprefs)
        r('tag_browser_show_tooltips', gprefs)
        r('tag_browser_allow_keyboard_focus', gprefs)
        r('tag_browser_show_counts', gprefs)
        r('tag_browser_item_padding', gprefs)
        r('show_avg_rating', config)
        r('show_links_in_tag_browser', gprefs)
        r('show_notes_in_tag_browser', gprefs)
        r('icons_on_right_in_tag_browser', gprefs)

        self.tb_display_model = TBDisplayedFields(self.gui.current_db, self.tb_display_order, category_icons=self.gui.tags_view.model().category_custom_icons)
        self.tb_display_model.dataChanged.connect(self.changed_signal)
        self.tb_display_order.setModel(self.tb_display_model)
        self.tb_reset_layout_button.clicked.connect(partial(reset_layout, self, model=self.tb_display_model))
        self.tb_export_layout_button.clicked.connect(partial(export_layout, self, model=self.tb_display_model))
        self.tb_import_layout_button.clicked.connect(partial(import_layout, self, model=self.tb_display_model))

        mu = partial(move_field_up, self.tb_display_order, self.tb_display_model)
        md = partial(move_field_down, self.tb_display_order, self.tb_display_model)
        self.tb_up_button.clicked.connect(partial(mu, use_kbd_modifiers=True))
        self.tb_down_button.clicked.connect(partial(md, use_kbd_modifiers=True))
        self.tb_display_order.set_movement_functions(mu, md)

    def lazy_initialize(self):
        self.tb_display_model.initialize()
        self.tb_focus_label.setVisible(self.opt_tag_browser_allow_keyboard_focus.isChecked())

    def restore_defaults(self, *args):
        LazyConfigWidgetBase.restore_defaults(self)
        self.tb_display_model.restore_defaults()

    def commit(self, *args):
        self.tb_display_model.commit()
        return LazyConfigWidgetBase.commit(self)

    def refresh_gui(self, gui):
        gui.tags_view.set_look_and_feel()



# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/cover_grid.ui'
#
# Created by: PyQt6 UI code generator 6.10.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_cover_grid_tab(object):
    def setupUi(self, cover_grid_tab):
        cover_grid_tab.setObjectName("cover_grid_tab")
        cover_grid_tab.resize(458, 483)
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.formLayout_2 = QtWidgets.QFormLayout(self.tab_5)
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_19 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_19.setWordWrap(True)
        self.label_19.setObjectName("label_19")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.label_19)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.formLayout_2.setItem(3, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem)
        self.opt_cover_grid_show_title = QtWidgets.QCheckBox(parent=self.tab_5)
        self.opt_cover_grid_show_title.setObjectName("opt_cover_grid_show_title")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_cover_grid_show_title)
        self.label_14 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_14.setObjectName("label_14")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_14)
        self.opt_field_under_covers_in_grid = QtWidgets.QComboBox(parent=self.tab_5)
        self.opt_field_under_covers_in_grid.setObjectName("opt_field_under_covers_in_grid")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_field_under_covers_in_grid)
        self.label_16 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_16.setObjectName("label_16")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_16)
        self.opt_cover_grid_spacing = QtWidgets.QDoubleSpinBox(parent=self.tab_5)
        self.opt_cover_grid_spacing.setDecimals(2)
        self.opt_cover_grid_spacing.setSingleStep(0.1)
        self.opt_cover_grid_spacing.setObjectName("opt_cover_grid_spacing")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_cover_grid_spacing)
        self.cg_background_box = BackgroundConfig(parent=self.tab_5)
        self.cg_background_box.setObjectName("cg_background_box")
        self.formLayout_2.setWidget(8, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.cg_background_box)
        self.groupBox_4 = QtWidgets.QGroupBox(parent=self.tab_5)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.cover_grid_larger_cover = QtWidgets.QPushButton(parent=self.groupBox_4)
        icon = QtGui.QIcon.ic("plus.png")
        self.cover_grid_larger_cover.setIcon(icon)
        self.cover_grid_larger_cover.setObjectName("cover_grid_larger_cover")
        self.gridLayout_4.addWidget(self.cover_grid_larger_cover, 1, 2, 1, 1)
        self.cover_grid_smaller_cover = QtWidgets.QPushButton(parent=self.groupBox_4)
        icon1 = QtGui.QIcon.ic("minus.png")
        self.cover_grid_smaller_cover.setIcon(icon1)
        self.cover_grid_smaller_cover.setObjectName("cover_grid_smaller_cover")
        self.gridLayout_4.addWidget(self.cover_grid_smaller_cover, 2, 2, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.groupBox_4)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 2, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.groupBox_4)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 1, 0, 1, 1)
        self.opt_cover_grid_height = QtWidgets.QDoubleSpinBox(parent=self.groupBox_4)
        self.opt_cover_grid_height.setDecimals(1)
        self.opt_cover_grid_height.setObjectName("opt_cover_grid_height")
        self.gridLayout_4.addWidget(self.opt_cover_grid_height, 2, 1, 1, 1)
        self.cover_grid_reset_size = QtWidgets.QPushButton(parent=self.groupBox_4)
        self.cover_grid_reset_size.setObjectName("cover_grid_reset_size")
        self.gridLayout_4.addWidget(self.cover_grid_reset_size, 1, 3, 2, 1)
        self.cover_grid_aspect_ratio = QtWidgets.QLabel(parent=self.groupBox_4)
        self.cover_grid_aspect_ratio.setText("")
        self.cover_grid_aspect_ratio.setObjectName("cover_grid_aspect_ratio")
        self.gridLayout_4.addWidget(self.cover_grid_aspect_ratio, 3, 0, 1, 5)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 1, 4, 1, 1)
        self.opt_cover_grid_width = QtWidgets.QDoubleSpinBox(parent=self.groupBox_4)
        self.opt_cover_grid_width.setDecimals(1)
        self.opt_cover_grid_width.setObjectName("opt_cover_grid_width")
        self.gridLayout_4.addWidget(self.opt_cover_grid_width, 1, 1, 1, 1)
        self.cover_size_label = QtWidgets.QLabel(parent=self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cover_size_label.sizePolicy().hasHeightForWidth())
        self.cover_size_label.setSizePolicy(sizePolicy)
        self.cover_size_label.setWordWrap(True)
        self.cover_size_label.setObjectName("cover_size_label")
        self.gridLayout_4.addWidget(self.cover_size_label, 0, 0, 1, 5)
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.groupBox_4)
        self.label_21 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_21.setObjectName("label_21")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_21)
        self.opt_emblem_size = QtWidgets.QSpinBox(parent=self.tab_5)
        self.opt_emblem_size.setMinimum(16)
        self.opt_emblem_size.setMaximum(1000)
        self.opt_emblem_size.setObjectName("opt_emblem_size")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_emblem_size)
        self.label_22 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_22.setObjectName("label_22")
        self.formLayout_2.setWidget(13, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_22)
        self.opt_emblem_position = QtWidgets.QComboBox(parent=self.tab_5)
        self.opt_emblem_position.setObjectName("opt_emblem_position")
        self.formLayout_2.setWidget(13, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_emblem_position)
        self.opt_cover_grid_text_flush_bottom = QtWidgets.QCheckBox(parent=self.tab_5)
        self.opt_cover_grid_text_flush_bottom.setObjectName("opt_cover_grid_text_flush_bottom")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_cover_grid_text_flush_bottom)
        cover_grid_tab.addTab(self.tab_5, "")
        self.opt_cover_grid_icon_rules = EditRulesWidget()
        self.opt_cover_grid_icon_rules.setObjectName("opt_cover_grid_icon_rules")
        cover_grid_tab.addTab(self.opt_cover_grid_icon_rules, "")
        self.config_cache = CoverCacheConfig()
        self.config_cache.setObjectName("config_cache")
        cover_grid_tab.addTab(self.config_cache, "")
        self.label_14.setBuddy(self.opt_field_under_covers_in_grid)
        self.label_16.setBuddy(self.opt_cover_grid_spacing)
        self.label_12.setBuddy(self.opt_cover_grid_height)
        self.label_11.setBuddy(self.opt_cover_grid_width)
        self.label_21.setBuddy(self.opt_emblem_size)
        self.label_22.setBuddy(self.opt_emblem_position)

        self.retranslateUi(cover_grid_tab)
        cover_grid_tab.setCurrentIndex(0)
        self.opt_cover_grid_show_title.toggled['bool'].connect(self.opt_field_under_covers_in_grid.setEnabled) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(cover_grid_tab)

    def retranslateUi(self, cover_grid_tab):

        self.label_19.setText(_("Control the Cover grid view. You can enable this view by clicking the \"Layout\" button in the bottom right corner of the main calibre window."))
        self.opt_cover_grid_show_title.setText(_("Show a &field (such as title) under the covers"))
        self.label_14.setText(_("Field to show &under the covers:"))
        self.label_16.setText(_("Spac&ing between covers:"))
        self.opt_cover_grid_spacing.setToolTip(_("The spacing between covers. A value of zero means calculate automatically based on cover size."))
        self.opt_cover_grid_spacing.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_spacing.setSuffix(_(" cm"))
        self.cg_background_box.setProperty("title", _("Background for the Cover grid"))
        self.groupBox_4.setTitle(_("Cover size"))
        self.cover_grid_larger_cover.setToolTip(_("Make the covers larger, maintaining current aspect ratio."))
        self.cover_grid_larger_cover.setText(_("&Larger covers"))
        self.cover_grid_smaller_cover.setToolTip(_("Make the covers smaller, maintaining current aspect ratio."))
        self.cover_grid_smaller_cover.setText(_("&Smaller covers"))
        self.label_12.setText(_("Cover &height: "))
        self.label_11.setText(_("Cover &width: "))
        self.opt_cover_grid_height.setToolTip(_("The height of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_height.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_height.setSuffix(_(" cm"))
        self.cover_grid_reset_size.setToolTip(_("Reset size to automatic"))
        self.cover_grid_reset_size.setText(_("&Reset size"))
        self.opt_cover_grid_width.setToolTip(_("The width of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_width.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_width.setSuffix(_(" cm"))
        self.cover_size_label.setText(_("By default, calibre chooses a cover size based on your computer\'s screen size. You can change the cover size here:"))
        self.label_21.setText(_("Size of the emblems (if any) shown &next to the covers: "))
        self.opt_emblem_size.setSuffix(_(" px"))
        self.label_22.setText(_("&Location of the emblems shown next to the covers:"))
        self.opt_cover_grid_text_flush_bottom.setToolTip(_("<p>By default, the text is drawn at the bottom of the cell, leaving a gap between text and cover when the cover is small or has a wide aspect ratio. This option instead causes the text to be drawn flush with the bottom of the cover."))
        self.opt_cover_grid_text_flush_bottom.setText(_("Draw text flush with the &bottom of the rendered cover"))
        cover_grid_tab.setTabText(cover_grid_tab.indexOf(self.tab_5), _("&Layout"))
        cover_grid_tab.setTabText(cover_grid_tab.indexOf(self.opt_cover_grid_icon_rules), _("&Emblems"))
        cover_grid_tab.setTabText(cover_grid_tab.indexOf(self.config_cache), _("&Performance"))
from calibre.gui2.preferences.look_feel_tabs import BackgroundConfig, CoverCacheConfig, EditRulesWidget



# License: GPLv3 Copyright: 2009, John Schember <john@nachtimwald.com>

"""
Interface defining the necessary public functions for a pdb format reader.
"""


class FormatReader:
    def __init__(self, header, stream, log, options):
        raise NotImplementedError()

    def extract_content(self, output_dir):
        raise NotImplementedError()



#!/usr/bin/env python
# License: GPLv3 Copyright: 2010, Kovid Goyal <kovid@kovidgoyal.net>

import json
from functools import partial

from qt.core import QListWidgetItem, Qt

from calibre.gui2 import choose_files, choose_save_file, error_dialog, gprefs
from calibre.gui2.preferences import LazyConfigWidgetBase
from calibre.gui2.preferences.look_feel_tabs import DisplayedFields
from calibre.gui2.preferences.look_feel_tabs.tb_hierarchy_ui import Ui_Form
from calibre.utils.localization import _


class TBHierarchicalFields(DisplayedFields):
    # The code in this class depends on the fact that the tag browser is
    # initialized before this class is instantiated.

    cant_make_hierarical = {'authors', 'publisher', 'formats', 'news', 'identifiers', 'languages', 'rating'}

    def __init__(self, db, parent=None, category_icons=None):
        DisplayedFields.__init__(self, db, parent, category_icons=category_icons)
        from calibre.gui2.ui import get_gui

        self.gui = get_gui(fail_if_absent=True)

    def initialize(self, use_defaults=False, pref_data_override=None):
        from calibre.gui2.tag_browser.model import TagsModel

        tv = self.gui.tags_view
        model = tv.model()
        assert isinstance(model, TagsModel)
        cats = [k for k in model.categories.keys() if (not k.startswith('@') and k not in self.cant_make_hierarical)]
        ans = []
        if use_defaults:
            ans = [[k, False] for k in cats]
            self.changed = True
        elif pref_data_override:
            ph = dict(pref_data_override)
            ans = [[k, ph.get(k, False)] for k in cats]
            self.changed = True
        else:
            hier_cats = self.db.prefs.get('categories_using_hierarchy') or ()
            for key in cats:
                ans.append([key, key in hier_cats])
        self.beginResetModel()
        self.fields = ans
        self.endResetModel()

    def commit(self):
        if self.changed:
            self.db.prefs.set('categories_using_hierarchy', [k for k, v in self.fields if v])


class TbHierarchyTab(LazyConfigWidgetBase, Ui_Form):
    def genesis(self, gui):
        self.gui = gui
        self.tb_search_order_up_button.clicked.connect(self.move_tb_search_up)
        self.tb_search_order_down_button.clicked.connect(self.move_tb_search_down)
        self.tb_search_order.set_movement_functions(self.move_tb_search_up, self.move_tb_search_down)
        self.tb_search_order_reset_button.clicked.connect(self.reset_tb_search_order)

    def lazy_initialize(self):
        self.fill_tb_search_order_box()

        self.tb_hierarchical_cats_model = TBHierarchicalFields(
            self.gui.current_db,
            self.tb_hierarchical_cats,
            category_icons=self.gui.tags_view.model().category_custom_icons,
        )
        self.tb_hierarchical_cats_model.dataChanged.connect(self.changed_signal)
        self.tb_hierarchical_cats.setModel(self.tb_hierarchical_cats_model)
        self.tb_hierarchy_reset_layout_button.clicked.connect(partial(self.reset_layout, model=self.tb_hierarchical_cats_model))
        self.tb_hierarchy_export_layout_button.clicked.connect(partial(self.export_layout, model=self.tb_hierarchical_cats_model))
        self.tb_hierarchy_import_layout_button.clicked.connect(partial(self.import_layout, model=self.tb_hierarchical_cats_model))
        self.tb_hierarchical_cats_model.initialize()

    def fill_tb_search_order_box(self):
        # The tb_search_order is a directed graph of nodes with an arc to the next
        # node in the sequence. Node 0 (zero) is the start node with the last node
        # arcing back to node 0. This code linearizes the graph

        choices = [
            (1, _('Search for books containing the current item')),
            (2, _('Search for books containing the current item or its children')),
            (3, _('Search for books not containing the current item')),
            (4, _('Search for books not containing the current item or its children')),
        ]
        icon_map = self.gui.tags_view.model().icon_state_map

        order = gprefs.get('tb_search_order')
        self.tb_search_order.clear()
        node = 0
        while True:
            v = order[str(node)]
            if v == 0:
                break
            item = QListWidgetItem(icon_map[v], choices[v - 1][1])
            item.setData(Qt.ItemDataRole.UserRole, choices[v - 1][0])
            self.tb_search_order.addItem(item)
            node = v

    def move_tb_search_up(self, *args, **kwargs):
        idx = self.tb_search_order.currentRow()
        if idx <= 0:
            return
        item = self.tb_search_order.takeItem(idx)
        self.tb_search_order.insertItem(idx - 1, item)
        self.tb_search_order.setCurrentRow(idx - 1)
        self.changed_signal.emit()

    def move_tb_search_down(self, *args, **kwargs):
        idx = self.tb_search_order.currentRow()
        if idx < 0 or idx == 3:
            return
        item = self.tb_search_order.takeItem(idx)
        self.tb_search_order.insertItem(idx + 1, item)
        self.tb_search_order.setCurrentRow(idx + 1)
        self.changed_signal.emit()

    def tb_search_order_commit(self):
        t = {}
        # Walk the items in the list box building the (node -> node) graph of
        # the option order
        node = 0
        for i in range(4):
            item = self.tb_search_order.item(i)
            assert item is not None
            v = item.data(Qt.ItemDataRole.UserRole)
            # JSON dumps converts integer keys to strings, so do it explicitly
            t[str(node)] = v
            node = v
        # Add the arc from the last node back to node 0
        t[str(node)] = 0
        gprefs.set('tb_search_order', t)

    def reset_tb_search_order(self):
        gprefs.set('tb_search_order', gprefs.defaults['tb_search_order'])
        self.fill_tb_search_order_box()
        self.changed_signal.emit()

    def reset_layout(self, model=None):
        assert model is not None
        model.initialize(use_defaults=True)
        self.changed_signal.emit()

    def export_layout(self, model=None):
        assert model is not None
        filename = choose_save_file(self, 'em_import_export_field_list', _('Save column list to file'), filters=[(_('Column list'), ['json'])])
        if filename:
            try:
                with open(filename, 'w') as f:
                    json.dump(model.fields, f, indent=1)
            except Exception as err:
                error_dialog(self, _('Export field layout'), _('<p>Could not write field list. Error:<br>%s') % err, show=True)

    def import_layout(self, model=None):
        assert model is not None
        filename = choose_files(self, 'em_import_export_field_list', _('Load column list from file'), filters=[(_('Column list'), ['json'])])
        if filename:
            try:
                with open(filename[0]) as f:
                    fields = json.load(f)
                model.initialize(pref_data_override=fields)
                self.changed_signal.emit()
            except Exception as err:
                error_dialog(self, _('Import layout'), _('<p>Could not read field list. Error:<br>%s') % err, show=True)

    def restore_defaults(self, *args):
        LazyConfigWidgetBase.restore_defaults(self)
        self.tb_hierarchical_cats_model.restore_defaults()
        self.reset_tb_search_order()

    def commit(self, *args):
        self.tb_search_order_commit()
        self.tb_hierarchical_cats_model.commit()
        return LazyConfigWidgetBase.commit(self)



#!/usr/bin/env python
# License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>

import sys

from calibre import prints
from calibre.db.legacy import LibraryDatabase
from calibre.utils.localization import _

readonly = False
version = 0  # change this if you change signature of implementation()
no_remote = True


def implementation(db, notify_changes, *args):
    raise NotImplementedError()


def option_parser(get_parser, args):
    parser = get_parser(
        _(
            '''\
%prog remove_custom_column [options] label

Remove the custom column identified by label. You can see available
columns with the custom_columns command.
    '''
        )
    )
    parser.add_option('-f', '--force', default=False, action='store_true', help=_('Do not ask for confirmation'))
    return parser


def input_unicode(prompt):
    ans = input(prompt)
    if isinstance(ans, bytes):
        ans = ans.decode(sys.stdin.encoding)
    return ans


def do_remove_custom_column(db, label, force):
    if not force:
        q = input_unicode(_('You will lose all data in the column: %s. Are you sure (y/n)? ') % label)
        if q.lower().strip() != _('y'):
            return
    try:
        db.delete_custom_column(label=label)
    except KeyError:
        raise SystemExit(_('No column named %s found. You must use column labels, not titles. Use calibredb custom_columns to get a list of labels.') % label)
    prints(f'Column {label!r} removed.')


def main(opts, args, dbctx):
    if len(args) < 1:
        raise SystemExit(_('Error: You must specify a column label'))

    do_remove_custom_column(dbctx.db, args[0], opts.force)
    # Update the stored field_metadata
    dbctx.db.close()
    db = LibraryDatabase(dbctx.db.library_path)
    m = db.field_metadata.all_metadata()
    db.new_api.set_pref('field_metadata', m)
    return 0



# License: GPLv3 Copyright: 2011, Kovid Goyal <kovid@kovidgoyal.net>


class ConversionUserFeedBack(Exception):
    def __init__(self, title, msg, level='info', det_msg=''):
        """Show a simple message to the user

        :param title: The title (very short description)
        :param msg: The message to show the user
        :param level: Must be one of 'info', 'warn' or 'error'
        :param det_msg: Optional detailed message to show the user
        """
        import json

        Exception.__init__(self, json.dumps({'msg': msg, 'level': level, 'det_msg': det_msg, 'title': title}))
        self.title, self.msg, self.det_msg = title, msg, det_msg
        self.level = level


# Ensure exception uses fully qualified name as this is used to detect it in
# the GUI.
ConversionUserFeedBack.__name__ = 'calibre.ebooks.conversion.ConversionUserFeedBack'



#!/usr/bin/env python
# License: GPLv3 Copyright: 2010, Kovid Goyal <kovid@kovidgoyal.net>

from calibre.gui2 import gprefs
from calibre.gui2.preferences import AbortCommit, ConfigWidgetBase, test_widget
from calibre.gui2.preferences.saving_ui import Ui_Form
from calibre.library.save_to_disk import config
from calibre.utils.config import ConfigProxy
from calibre.utils.localization import _


class ConfigWidget(ConfigWidgetBase, Ui_Form):
    def genesis(self, gui):
        self.gui = gui
        self.proxy = ConfigProxy(config())

        r = self.register

        for x in (
            'asciiize',
            'update_metadata',
            'save_cover',
            'write_opf',
            'replace_whitespace',
            'to_lowercase',
            'formats',
            'timefmt',
            'save_extra_files',
        ):
            r(x, self.proxy)
        r('show_files_after_save', gprefs)

        self.save_template.changed_signal.connect(self.changed_signal.emit)

    def initialize(self):
        ConfigWidgetBase.initialize(self)
        self.save_template.blockSignals(True)
        self.save_template.initialize(
            'save_to_disk',
            self.proxy['template'],
            self.proxy.help('template'),
            self.gui.library_view.model().db.field_metadata,
        )
        self.opt_timefmt.setToolTip(
            '<p>' + self.opt_timefmt.toolTip() + '</p><p>' + _('Changes will not appear in the template editor until you press the apply button.') + '</p>'
        )
        self.save_template.blockSignals(False)

    def restore_defaults(self, *args):
        ConfigWidgetBase.restore_defaults(self)
        self.save_template.set_value(self.proxy.defaults['template'])

    def commit(self, *args):
        if not self.save_template.validate():
            raise AbortCommit('abort')
        self.save_template.save_settings(self.proxy, 'template')
        return ConfigWidgetBase.commit(self)

    def refresh_gui(self, gui):
        gui.iactions['Save To Disk'].reread_prefs()
        # Ensure worker process reads updated settings
        gui.spare_pool().shutdown()


if __name__ == '__main__':
    from qt.core import QApplication

    app = QApplication([])
    test_widget('Import/Export', 'Saving')
